@@ -11,6 +11,9 @@ use crate::{
     PredicateError,
 };
 
+#[cfg(feature = "unstable-debug-counters")]
+use crate::sync::debug_counters::CacheDebugStats;
+
 use crossbeam_channel::{Sender, TrySendError};
 use std::{
     any::TypeId,
@@ -27,15 +30,13 @@ use std::{
 /// `Cache` supports full concurrency of retrievals and a high expected concurrency
 /// for updates. It can be accessed inside and outside of asynchronous contexts.
 ///
-/// `Cache` utilizes a lock-free concurrent hash table `SegmentedHashMap` from the
-/// [moka-cht][moka-cht-crate] crate for the central key-value storage. `Cache`
-/// performs a best-effort bounding of the map using an entry replacement algorithm
-/// to determine which entries to evict when the capacity is exceeded.
+/// `Cache` utilizes a lock-free concurrent hash table as the central key-value
+/// storage. `Cache` performs a best-effort bounding of the map using an entry
+/// replacement algorithm to determine which entries to evict when the capacity is
+/// exceeded.
 ///
 /// To use this cache, enable a crate feature called "future".
 ///
-/// [moka-cht-crate]: https://crates.io/crates/moka-cht
-///
 /// # Examples
 ///
 /// Cache entries are manually added using an insert method, and are stored in the
@@ -695,6 +696,11 @@ where
         1
     }
 
+    #[cfg(feature = "unstable-debug-counters")]
+    pub fn debug_stats(&self) -> CacheDebugStats {
+        self.base.debug_stats()
+    }
+
     #[cfg(test)]
     fn estimated_entry_count(&self) -> u64 {
         self.base.estimated_entry_count()
