@@ -4,17 +4,22 @@ use crate::common::{deque::DeqNode, time::Instant};
 
 use parking_lot::Mutex;
 use std::{ptr::NonNull, sync::Arc};
+use tagptr::TagNonNull;
+use triomphe::Arc as TrioArc;
 
 pub(crate) mod base_cache;
 mod builder;
 mod cache;
 mod deques;
-mod entry_info;
+pub(crate) mod entry_info;
 pub(crate) mod housekeeper;
 mod invalidator;
 mod segment;
 mod value_initializer;
 
+#[cfg(feature = "unstable-debug-counters")]
+pub(crate) mod debug_counters;
+
 pub use builder::CacheBuilder;
 pub use cache::Cache;
 pub use segment::SegmentedCache;
@@ -68,14 +73,14 @@ impl<K> Clone for KeyHash<K> {
 
 pub(crate) struct KeyDate<K> {
     key: Arc<K>,
-    entry_info: EntryInfo,
+    entry_info: TrioArc<EntryInfo>,
 }
 
 impl<K> KeyDate<K> {
-    pub(crate) fn new(key: Arc<K>, entry_info: &EntryInfo) -> Self {
+    pub(crate) fn new(key: Arc<K>, entry_info: &TrioArc<EntryInfo>) -> Self {
         Self {
             key,
-            entry_info: entry_info.clone(),
+            entry_info: TrioArc::clone(entry_info),
         }
     }
 
@@ -91,15 +96,15 @@ impl<K> KeyDate<K> {
 pub(crate) struct KeyHashDate<K> {
     key: Arc<K>,
     hash: u64,
-    entry_info: EntryInfo,
+    entry_info: TrioArc<EntryInfo>,
 }
 
 impl<K> KeyHashDate<K> {
-    pub(crate) fn new(kh: KeyHash<K>, entry_info: &EntryInfo) -> Self {
+    pub(crate) fn new(kh: KeyHash<K>, entry_info: &TrioArc<EntryInfo>) -> Self {
         Self {
             key: kh.key,
             hash: kh.hash,
-            entry_info: entry_info.clone(),
+            entry_info: TrioArc::clone(entry_info),
         }
     }
 
@@ -114,11 +119,11 @@ impl<K> KeyHashDate<K> {
 
 pub(crate) struct KvEntry<K, V> {
     pub(crate) key: Arc<K>,
-    pub(crate) entry: Arc<ValueEntry<K, V>>,
+    pub(crate) entry: TrioArc<ValueEntry<K, V>>,
 }
 
 impl<K, V> KvEntry<K, V> {
-    pub(crate) fn new(key: Arc<K>, entry: Arc<ValueEntry<K, V>>) -> Self {
+    pub(crate) fn new(key: Arc<K>, entry: TrioArc<ValueEntry<K, V>>) -> Self {
         Self { key, entry }
     }
 }
@@ -168,12 +173,12 @@ impl<K> AccessTime for DeqNode<KeyHashDate<K>> {
 }
 
 // DeqNode for an access order queue.
-type KeyDeqNodeAo<K> = NonNull<DeqNode<KeyHashDate<K>>>;
+type KeyDeqNodeAo<K> = TagNonNull<DeqNode<KeyHashDate<K>>, 2>;
 
 // DeqNode for the write order queue.
 type KeyDeqNodeWo<K> = NonNull<DeqNode<KeyDate<K>>>;
 
-struct DeqNodes<K> {
+pub(crate) struct DeqNodes<K> {
     access_order_q_node: Option<KeyDeqNodeAo<K>>,
     write_order_q_node: Option<KeyDeqNodeWo<K>>,
 }
@@ -183,12 +188,15 @@ unsafe impl<K> Send for DeqNodes<K> {}
 
 pub(crate) struct ValueEntry<K, V> {
     pub(crate) value: V,
-    info: EntryInfo,
+    info: TrioArc<EntryInfo>,
     nodes: Mutex<DeqNodes<K>>,
 }
 
 impl<K, V> ValueEntry<K, V> {
-    fn new(value: V, entry_info: EntryInfo) -> Self {
+    fn new(value: V, entry_info: TrioArc<EntryInfo>) -> Self {
+        #[cfg(feature = "unstable-debug-counters")]
+        self::debug_counters::InternalGlobalDebugCounters::value_entry_created();
+
         Self {
             value,
             info: entry_info,
@@ -199,7 +207,10 @@ impl<K, V> ValueEntry<K, V> {
         }
     }
 
-    fn new_from(value: V, entry_info: EntryInfo, other: &Self) -> Self {
+    fn new_from(value: V, entry_info: TrioArc<EntryInfo>, other: &Self) -> Self {
+        #[cfg(feature = "unstable-debug-counters")]
+        self::debug_counters::InternalGlobalDebugCounters::value_entry_created();
+
         let nodes = {
             let other_nodes = other.nodes.lock();
             DeqNodes {
@@ -218,7 +229,7 @@ impl<K, V> ValueEntry<K, V> {
         }
     }
 
-    pub(crate) fn entry_info(&self) -> &EntryInfo {
+    pub(crate) fn entry_info(&self) -> &TrioArc<EntryInfo> {
         &self.info
     }
 
@@ -266,7 +277,14 @@ impl<K, V> ValueEntry<K, V> {
     }
 }
 
-impl<K, V> AccessTime for Arc<ValueEntry<K, V>> {
+#[cfg(feature = "unstable-debug-counters")]
+impl<K, V> Drop for ValueEntry<K, V> {
+    fn drop(&mut self) {
+        self::debug_counters::InternalGlobalDebugCounters::value_entry_dropped();
+    }
+}
+
+impl<K, V> AccessTime for TrioArc<ValueEntry<K, V>> {
     #[inline]
     fn last_accessed(&self) -> Option<Instant> {
         self.info.last_accessed()
@@ -288,56 +306,16 @@ impl<K, V> AccessTime for Arc<ValueEntry<K, V>> {
     }
 }
 
-#[derive(Clone, Copy, Debug)]
-pub(crate) enum CacheFeatures {
-    Plain,
-    Weighted,
-}
-
-impl CacheFeatures {
-    pub(crate) fn new(is_weighter_defined: bool) -> Self {
-        if is_weighter_defined {
-            Self::Weighted
-        } else {
-            Self::Plain
-        }
-    }
-}
-
-pub(crate) struct ValueEntryBuilder(CacheFeatures);
-
-impl ValueEntryBuilder {
-    pub(crate) fn new(features: CacheFeatures) -> Self {
-        Self(features)
-    }
-
-    pub(crate) fn build<K, V>(&self, value: V, policy_weight: u32) -> ValueEntry<K, V> {
-        let info = EntryInfo::new(self.0, policy_weight);
-        ValueEntry::new(value, info)
-    }
-
-    pub(crate) fn build_from<K, V>(
-        &self,
-        value: V,
-        policy_weight: u32,
-        other: &ValueEntry<K, V>,
-    ) -> ValueEntry<K, V> {
-        let info = other.info.clone();
-        info.set_policy_weight(policy_weight);
-        ValueEntry::new_from(value, info, other)
-    }
-}
-
 pub(crate) enum ReadOp<K, V> {
     // u64 is the hash of the key.
-    Hit(u64, Arc<ValueEntry<K, V>>, Instant),
+    Hit(u64, TrioArc<ValueEntry<K, V>>, Instant),
     Miss(u64),
 }
 
 pub(crate) enum WriteOp<K, V> {
     Upsert {
         key_hash: KeyHash<K>,
-        value_entry: Arc<ValueEntry<K, V>>,
+        value_entry: TrioArc<ValueEntry<K, V>>,
         old_weight: u32,
         new_weight: u32,
     },
