@@ -1,17 +1,23 @@
 use super::{
     deques::Deques,
+    entry_info::EntryInfo,
     housekeeper::{Housekeeper, InnerSync, SyncPace},
     invalidator::{GetOrRemoveEntry, InvalidationResult, Invalidator, KeyDateLite, PredicateFun},
-    AccessTime, CacheFeatures, KeyDate, KeyHash, KeyHashDate, KvEntry, PredicateId, ReadOp,
-    ValueEntry, ValueEntryBuilder, Weigher, WriteOp,
+    AccessTime, KeyDate, KeyHash, KeyHashDate, KvEntry, PredicateId, ReadOp, ValueEntry, Weigher,
+    WriteOp,
 };
+
+#[cfg(feature = "unstable-debug-counters")]
+use super::debug_counters::CacheDebugStats;
+
 use crate::{
     common::{
         self,
         atomic_time::AtomicInstant,
-        deque::{CacheRegion, DeqNode, Deque},
+        deque::{DeqNode, Deque},
         frequency_sketch::FrequencySketch,
         time::{CheckedTimeOps, Clock, Instant},
+        CacheRegion,
     },
     PredicateError,
 };
@@ -31,6 +37,7 @@ use std::{
     },
     time::Duration,
 };
+use triomphe::Arc as TrioArc;
 
 pub(crate) const MAX_SYNC_REPEATS: usize = 4;
 
@@ -213,6 +220,11 @@ where
         self.inner.time_to_idle()
     }
 
+    #[cfg(feature = "unstable-debug-counters")]
+    pub fn debug_stats(&self) -> CacheDebugStats {
+        self.inner.debug_stats()
+    }
+
     #[cfg(test)]
     pub(crate) fn estimated_entry_count(&self) -> u64 {
         self.inner.estimated_entry_count()
@@ -252,7 +264,7 @@ where
         let mut op1 = None;
         let mut op2 = None;
 
-        // Since the cache (moka-cht::SegmentedHashMap) employs optimistic locking
+        // Since the cache (cht::SegmentedHashMap) employs optimistic locking
         // strategy, insert_with_or_modify() may get an insert/modify operation
         // conflicted with other concurrent hash table operations. In that case, it
         // has to retry the insertion or modification, so on_insert and/or on_modify
@@ -270,7 +282,7 @@ where
                     cnt,
                     WriteOp::Upsert {
                         key_hash: KeyHash::new(Arc::clone(&key), hash),
-                        value_entry: Arc::clone(&entry),
+                        value_entry: TrioArc::clone(&entry),
                         old_weight: 0,
                         new_weight: weight,
                     },
@@ -289,10 +301,10 @@ where
                 let cnt = op_cnt2.fetch_add(1, Ordering::Relaxed);
                 op2 = Some((
                     cnt,
-                    Arc::clone(old_entry),
+                    TrioArc::clone(old_entry),
                     WriteOp::Upsert {
                         key_hash: KeyHash::new(Arc::clone(&key), hash),
-                        value_entry: Arc::clone(&entry),
+                        value_entry: TrioArc::clone(&entry),
                         old_weight,
                         new_weight: weight,
                     },
@@ -320,8 +332,9 @@ where
     }
 
     #[inline]
-    fn new_value_entry(&self, value: V, policy_weight: u32) -> Arc<ValueEntry<K, V>> {
-        Arc::new(self.inner.value_entry_builder.build(value, policy_weight))
+    fn new_value_entry(&self, value: V, policy_weight: u32) -> TrioArc<ValueEntry<K, V>> {
+        let info = TrioArc::new(EntryInfo::new(policy_weight));
+        TrioArc::new(ValueEntry::new(value, info))
     }
 
     #[inline]
@@ -330,12 +343,10 @@ where
         value: V,
         policy_weight: u32,
         other: &ValueEntry<K, V>,
-    ) -> Arc<ValueEntry<K, V>> {
-        Arc::new(
-            self.inner
-                .value_entry_builder
-                .build_from(value, policy_weight, other),
-        )
+    ) -> TrioArc<ValueEntry<K, V>> {
+        let info = TrioArc::clone(other.entry_info());
+        info.set_policy_weight(policy_weight);
+        TrioArc::new(ValueEntry::new_from(value, info, other))
     }
 
     #[inline]
@@ -384,7 +395,7 @@ where
             }
         }
         // Enable the frequency sketch.
-        self.inner.enable_frequency_sketch();
+        self.inner.enable_frequency_sketch_for_testing();
     }
 
     pub(crate) fn set_expiration_clock(&self, clock: Option<Clock>) {
@@ -457,11 +468,9 @@ enum AdmissionResult<K> {
     },
 }
 
-type CacheStore<K, V, S> = moka_cht::SegmentedHashMap<Arc<K>, Arc<ValueEntry<K, V>>, S>;
+type CacheStore<K, V, S> = crate::cht::SegmentedHashMap<Arc<K>, TrioArc<ValueEntry<K, V>>, S>;
 
-type CacheEntry<K, V> = (Arc<K>, Arc<ValueEntry<K, V>>);
-
-// type BoxedValueEntryBuilder<K, V> = Box<dyn ValueEntryBuilder<K, V> + Send + Sync + 'static>;
+type CacheEntry<K, V> = (Arc<K>, TrioArc<ValueEntry<K, V>>);
 
 pub(crate) struct Inner<K, V, S> {
     max_capacity: Option<u64>,
@@ -469,9 +478,9 @@ pub(crate) struct Inner<K, V, S> {
     weighted_size: AtomicCell<u64>,
     cache: CacheStore<K, V, S>,
     build_hasher: S,
-    value_entry_builder: ValueEntryBuilder,
     deques: Mutex<Deques<K>>,
     frequency_sketch: RwLock<FrequencySketch>,
+    frequency_sketch_enabled: AtomicBool,
     read_op_ch: Receiver<ReadOp<K, V>>,
     write_op_ch: Receiver<WriteOp<K, V>>,
     time_to_live: Option<Duration>,
@@ -509,24 +518,21 @@ where
             .map(|cap| cap + WRITE_LOG_SIZE * 4)
             .unwrap_or_default();
         let num_segments = 64;
-        let cache = moka_cht::SegmentedHashMap::with_num_segments_capacity_and_hasher(
+        let cache = crate::cht::SegmentedHashMap::with_num_segments_capacity_and_hasher(
             num_segments,
             initial_capacity,
             build_hasher.clone(),
         );
 
-        let features = CacheFeatures::new(weigher.is_some());
-        let value_entry_builder = ValueEntryBuilder::new(features);
-
         Self {
             max_capacity: max_capacity.map(|n| n as u64),
             entry_count: Default::default(),
             weighted_size: Default::default(),
             cache,
             build_hasher,
-            value_entry_builder,
             deques: Mutex::new(Default::default()),
             frequency_sketch: RwLock::new(Default::default()),
+            frequency_sketch_enabled: Default::default(),
             read_op_ch,
             write_op_ch,
             time_to_live,
@@ -590,6 +596,19 @@ where
         self.time_to_idle
     }
 
+    #[cfg(feature = "unstable-debug-counters")]
+    pub fn debug_stats(&self) -> CacheDebugStats {
+        let ec = self.entry_count.load();
+        let ws = self.weighted_size.load();
+
+        CacheDebugStats::new(
+            ec,
+            ws,
+            (self.cache.capacity() * 2) as u64,
+            self.frequency_sketch.read().table_size(),
+        )
+    }
+
     #[cfg(test)]
     #[inline]
     fn estimated_entry_count(&self) -> u64 {
@@ -641,7 +660,7 @@ where
     }
 
     #[inline]
-    fn is_invalidated_entry(&self, key: &Arc<K>, entry: &Arc<ValueEntry<K, V>>) -> bool {
+    fn is_invalidated_entry(&self, key: &Arc<K>, entry: &TrioArc<ValueEntry<K, V>>) -> bool {
         if self.invalidator_enabled {
             if let Some(inv) = &*self.invalidator.read() {
                 return inv.apply_predicates(key, entry);
@@ -676,18 +695,34 @@ where
     K: Hash + Eq,
     S: BuildHasher,
 {
-    fn get_value_entry(&self, key: &Arc<K>) -> Option<Arc<ValueEntry<K, V>>> {
+    fn get_value_entry(&self, key: &Arc<K>) -> Option<TrioArc<ValueEntry<K, V>>> {
         self.cache.get(key)
     }
 
-    fn remove_key_value_if<F>(&self, key: &Arc<K>, condition: F) -> Option<Arc<ValueEntry<K, V>>>
+    fn remove_key_value_if<F>(
+        &self,
+        key: &Arc<K>,
+        condition: F,
+    ) -> Option<TrioArc<ValueEntry<K, V>>>
     where
-        F: FnMut(&Arc<K>, &Arc<ValueEntry<K, V>>) -> bool,
+        F: FnMut(&Arc<K>, &TrioArc<ValueEntry<K, V>>) -> bool,
     {
         self.cache.remove_if(key, condition)
     }
 }
 
+#[cfg(feature = "unstable-debug-counters")]
+mod batch_size {
+    pub(crate) const EVICTION_BATCH_SIZE: usize = 10_000;
+    pub(crate) const INVALIDATION_BATCH_SIZE: usize = 10_000;
+}
+
+#[cfg(not(feature = "unstable-debug-counters"))]
+mod batch_size {
+    pub(crate) const EVICTION_BATCH_SIZE: usize = 500;
+    pub(crate) const INVALIDATION_BATCH_SIZE: usize = 500;
+}
+
 // TODO: Divide this method into smaller methods so that unit tests can do more
 // precise testing.
 // - sync_reads
@@ -701,9 +736,6 @@ where
     S: BuildHasher + Clone + Send + Sync + 'static,
 {
     fn sync(&self, max_repeats: usize) -> Option<SyncPace> {
-        const EVICTION_BATCH_SIZE: usize = 500;
-        const INVALIDATION_BATCH_SIZE: usize = 500;
-
         let mut deqs = self.deques.lock();
         let mut calls = 0;
         let mut should_sync = true;
@@ -723,7 +755,9 @@ where
                 self.apply_writes(&mut deqs, w_len, &mut counters);
             }
 
-            self.enable_frequency_sketch_if_needed(&counters);
+            if self.should_enable_frequency_sketch(&counters) {
+                self.enable_frequency_sketch(&counters);
+            }
 
             calls += 1;
             should_sync = self.read_op_ch.len() >= READ_LOG_FLUSH_POINT
@@ -731,7 +765,7 @@ where
         }
 
         if self.has_expiry() || self.has_valid_after() {
-            self.evict_expired(&mut deqs, EVICTION_BATCH_SIZE, &mut counters);
+            self.evict_expired(&mut deqs, batch_size::EVICTION_BATCH_SIZE, &mut counters);
         }
 
         if self.invalidator_enabled {
@@ -740,7 +774,7 @@ where
                     self.invalidate_entries(
                         invalidator,
                         &mut deqs,
-                        INVALIDATION_BATCH_SIZE,
+                        batch_size::INVALIDATION_BATCH_SIZE,
                         &mut counters,
                     );
                 }
@@ -752,7 +786,7 @@ where
         if weights_to_evict > 0 {
             self.evict_lru_entries(
                 &mut deqs,
-                EVICTION_BATCH_SIZE,
+                batch_size::EVICTION_BATCH_SIZE,
                 weights_to_evict,
                 &mut counters,
             );
@@ -796,30 +830,41 @@ where
     }
 
     #[inline]
-    fn enable_frequency_sketch_if_needed(&self, counters: &EvictionCounters) {
-        if let Some(max_cap) = self.max_capacity {
-            if counters.weighted_size >= max_cap / 2 {
-                self.do_enable_frequency_sketch(counters.entry_count, max_cap);
-            }
-        }
-    }
-
-    #[cfg(test)]
-    fn enable_frequency_sketch(&self) {
-        if let Some(max_cap) = self.max_capacity {
-            self.do_enable_frequency_sketch(self.entry_count.load(), max_cap);
+    fn should_enable_frequency_sketch(&self, counters: &EvictionCounters) -> bool {
+        if self.frequency_sketch_enabled.load(Ordering::Acquire) {
+            false
+        } else if let Some(max_cap) = self.max_capacity {
+            counters.weighted_size >= max_cap / 2
+        } else {
+            false
         }
     }
 
     #[inline]
-    fn do_enable_frequency_sketch(&self, entry_count: u64, max_capacity: u64) {
-        let num_entries = if self.weigher.is_some() {
-            entry_count * 2
-        } else {
-            max_capacity
-        };
-        let skt_capacity = common::sketch_capacity(num_entries);
+    fn enable_frequency_sketch(&self, counters: &EvictionCounters) {
+        if let Some(max_cap) = self.max_capacity {
+            let c = counters;
+            let cap = if self.weigher.is_none() {
+                max_cap
+            } else {
+                (c.entry_count as f64 * (c.weighted_size as f64 / max_cap as f64)) as u64
+            };
+            self.do_enable_frequency_sketch(cap);
+        }
+    }
+
+    #[cfg(test)]
+    fn enable_frequency_sketch_for_testing(&self) {
+        if let Some(max_cap) = self.max_capacity {
+            self.do_enable_frequency_sketch(max_cap);
+        }
+    }
+
+    #[inline]
+    fn do_enable_frequency_sketch(&self, cache_capacity: u64) {
+        let skt_capacity = common::sketch_capacity(cache_capacity);
         self.frequency_sketch.write().ensure_capacity(skt_capacity);
+        self.frequency_sketch_enabled.store(true, Ordering::Release);
     }
 
     fn apply_reads(&self, deqs: &mut Deques<K>, count: usize) {
@@ -867,7 +912,7 @@ where
     fn handle_upsert(
         &self,
         kh: KeyHash<K>,
-        entry: Arc<ValueEntry<K, V>>,
+        entry: TrioArc<ValueEntry<K, V>>,
         old_weight: u32,
         new_weight: u32,
         timestamp: Instant,
@@ -1027,7 +1072,7 @@ where
     fn handle_admit(
         &self,
         kh: KeyHash<K>,
-        entry: &Arc<ValueEntry<K, V>>,
+        entry: &TrioArc<ValueEntry<K, V>>,
         policy_weight: u32,
         deqs: &mut Deques<K>,
         counters: &mut EvictionCounters,
@@ -1047,32 +1092,36 @@ where
 
     fn handle_remove(
         deqs: &mut Deques<K>,
-        entry: Arc<ValueEntry<K, V>>,
+        entry: TrioArc<ValueEntry<K, V>>,
         counters: &mut EvictionCounters,
     ) {
         if entry.is_admitted() {
             entry.set_is_admitted(false);
             counters.saturating_sub(1, entry.policy_weight());
+            // The following two unlink_* functions will unset the deq nodes.
             deqs.unlink_ao(&entry);
             Deques::unlink_wo(&mut deqs.write_order, &entry);
+        } else {
+            entry.unset_q_nodes();
         }
-        entry.unset_q_nodes();
     }
 
     fn handle_remove_with_deques(
         ao_deq_name: &str,
         ao_deq: &mut Deque<KeyHashDate<K>>,
         wo_deq: &mut Deque<KeyDate<K>>,
-        entry: Arc<ValueEntry<K, V>>,
+        entry: TrioArc<ValueEntry<K, V>>,
         counters: &mut EvictionCounters,
     ) {
         if entry.is_admitted() {
             entry.set_is_admitted(false);
             counters.saturating_sub(1, entry.policy_weight());
+            // The following two unlink_* functions will unset the deq nodes.
             Deques::unlink_ao_from_deque(ao_deq_name, ao_deq, &entry);
             Deques::unlink_wo(wo_deq, &entry);
+        } else {
+            entry.unset_q_nodes();
         }
-        entry.unset_q_nodes();
     }
 
     fn evict_expired(
@@ -1454,7 +1503,7 @@ mod tests {
                 None,
                 false,
             );
-            cache.inner.enable_frequency_sketch();
+            cache.inner.enable_frequency_sketch_for_testing();
             assert_eq!(
                 cache.inner.frequency_sketch.read().table_len(),
                 len as usize,
