@@ -21,12 +21,10 @@ use std::{
 /// `Cache` supports full concurrency of retrievals and a high expected concurrency
 /// for updates.
 ///
-/// `Cache` utilizes a lock-free concurrent hash table `SegmentedHashMap` from the
-/// [moka-cht][moka-cht-crate] crate for the central key-value storage. `Cache`
-/// performs a best-effort bounding of the map using an entry replacement algorithm
-/// to determine which entries to evict when the capacity is exceeded.
-///
-/// [moka-cht-crate]: https://crates.io/crates/moka-cht
+/// `Cache` utilizes a lock-free concurrent hash table as the central key-value
+/// storage. `Cache` performs a best-effort bounding of the map using an entry
+/// replacement algorithm to determine which entries to evict when the capacity is
+/// exceeded.
 ///
 /// # Examples
 ///
@@ -892,6 +890,8 @@ mod tests {
         let num_threads = 4;
         let cache = Cache::new(100);
 
+        // https://rust-lang.github.io/rust-clippy/master/index.html#needless_collect
+        #[allow(clippy::needless_collect)]
         let handles = (0..num_threads)
             .map(|id| {
                 let cache = cache.clone();
